
setupCF <- function(x,xvar,yr,fd=FALSE) {

  # Check for each possible xvar; add appropriate newdata row

  # Create scenario names
  
  # Return new data and scenario names
  k <- length(xvar)
  meandata <- apply(x,2,mean)

  # Ensure logical consistency
  if (any(xvar=="cmXfisrev")) {
    curxrow <- (1:k)[xvar=="cmXfisrev"]
    cur1row <- (1:k)[xvar=="cntminexp"]
    cur2row <- (1:k)[xvar=="fisrevenue"]
    meandata[curxrow] <- meandata[cur1row]*meandata[cur2row]
  }

  if (any(xvar=="cmXgdp")) {
    curxrow <- (1:k)[xvar=="cmXgdp"]
    cur1row <- (1:k)[xvar=="cntminexp"]
    cur2row <- (1:k)[xvar=="gdp"]
    meandata[curxrow] <- meandata[cur1row]*meandata[cur2row]
  }
  
  if (any(xvar=="apptage")) {
    curxrow <- (1:k)[xvar=="apptage"]
    if (meandata[curxrow]>=63) {
      if (any(xvar=="agege63")) {
        currow <- (1:k)[xvar=="agege63"]   
        meandata[currow] <- 1
      }
    } else {
      if (any(xvar=="agege63")) {
        currow <- (1:k)[xvar=="agege63"]   
        meandata[currow] <- 0
      }
    }

    meanage <- mean(x[,curxrow])

    if (any(xvar=="partyexp")) {
      partyexp <- mean(x[,(1:k)[xvar=="partyexp"]])
      implicitPtime <- yr - (meanage*partyexp)
      if (any(xvar=="ccp1935")) {
        currow <-  (1:k)[xvar=="ccp1935"]
        if (implicitPtime<=1935) {       
          meandata[currow] <- 1
        } else {
          meandata[currow] <- 0
        }
      }
      
      if (any(xvar=="ccp1945")) {
        currow <-  (1:k)[xvar=="ccp1945"]
        if (implicitPtime<=1945) {       
          meandata[currow] <- 1
        } else {
          meandata[currow] <- 0
        }
      }
      
      if (any(xvar=="ccp1949")) {
        currow <-  (1:k)[xvar=="ccp1949"]
        if (implicitPtime<=1949) {       
          meandata[currow] <- 1
        } else {
          meandata[currow] <- 0
        }
      }    
    }    
  }

  if (any(xvar=="partyyears")) {
    implicitPtime <- yr - mean(x[,(1:k)[xvar=="partyyears"]])
    if (any(xvar=="ccp1935")) {
      currow <-  (1:k)[xvar=="ccp1935"]
      if (implicitPtime<=1935) {       
        meandata[currow] <- 1
      } else {
        meandata[currow] <- 0
      }
    }
    
    if (any(xvar=="ccp1945")) {
      currow <-  (1:k)[xvar=="ccp1945"]
      if (implicitPtime<=1945) {       
        meandata[currow] <- 1
      } else {
        meandata[currow] <- 0
      }
    }
    
    if (any(xvar=="ccp1949")) {
      currow <-  (1:k)[xvar=="ccp1949"]
      if (implicitPtime<=1949) {       
        meandata[currow] <- 1
      } else {
        meandata[currow] <- 0
      }
    }    
  }
    
  newdata <- NULL
  newpredata <- NULL
  scenarios <- NULL

  
  scenvar <- "female"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- 1
    newprescen <- meandata
    newprescen[curxrow] <- 0
    scenname <- "Female"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)

    if (!fd) {
      newscen <- meandata
      newscen[curxrow] <- 0
      newprescen <- meandata
      newprescen[curxrow] <- 1
      scenname <- "Male"
      newdata <- rbind(newdata,newscen)
      newpredata <- rbind(newpredata,newprescen)
      scenarios <- c(scenarios,scenname)
    }
  }


  scenvar <- "nonhan"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- 1
    newprescen <- meandata
    newprescen[curxrow] <- 0
    scenname <- "Minority"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)

    if (!fd) {
      newscen <- meandata
      newscen[curxrow] <- 0
      newprescen <- meandata
      newprescen[curxrow] <- 1
      scenname <- "Han"
      newdata <- rbind(newdata,newscen)
      newpredata <- rbind(newpredata,newprescen)
      scenarios <- c(scenarios,scenname)
    }
  }

  
  scenvar <- "prince"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- 1
    newprescen <- meandata
    newprescen[curxrow] <- 0
    scenname <- "Princeling"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)

    if (!fd) {
      newscen <- meandata
      newscen[curxrow] <- 0
      newprescen <- meandata
      newprescen[curxrow] <- 1
      scenname <- "Non-Princeling"
      newdata <- rbind(newdata,newscen)
      newpredata <- rbind(newpredata,newprescen)
      scenarios <- c(scenarios,scenname)
    }
  }


  scenvar <- "apptage"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- newscen[curxrow] + sd(x[,curxrow])
    newprescen <- meandata
    scenname <- "Age +1 sd"

    if (any(xvar=="agege63")) {
      currow <- (1:k)[xvar=="agege63"]
      if (newscen[curxrow]>=63) {
        newscen[currow] <- 1
      } else {
        newscen[currow] <- 0
      }

      if (newprescen[curxrow]>=63) {
        newprescen[currow] <- 1
      } else {
        newprescen[currow] <- 0
      }
    }

    if (any(xvar=="partyexp")) {
      scenage <- newscen[curxrow]
      preage <- newprescen[curxrow]
      partyexp <- newscen[(1:k)[xvar=="partyexp"]]
      partyexppre <- newprescen[(1:k)[xvar=="partyexp"]]
      implicitPtime <- yr - (scenage*partyexp)
      implicitprePtime <- yr - (preage*partyexppre)
      
      if (any(xvar=="ccp1935")) {
        currow <-  (1:k)[xvar=="ccp1935"]
        if (implicitPtime<=1935) {       
          newscen[currow] <- 1
        } else {
          newscen[currow] <- 0
        }
        if (implicitprePtime<=1935) {       
          newprescen[currow] <- 1
        } else {
          newprescen[currow] <- 0
        }
      }
      
      if (any(xvar=="ccp1945")) {
        currow <-  (1:k)[xvar=="ccp1945"]
        if (implicitPtime<=1945) {       
          newscen[currow] <- 1
        } else {
          newscen[currow] <- 0
        }
        if (implicitprePtime<=1945) {       
          newprescen[currow] <- 1
        } else {
          newprescen[currow] <- 0
        }
      }

      if (any(xvar=="ccp1949")) {
        currow <-  (1:k)[xvar=="ccp1949"]
        if (implicitPtime<=1949) {       
          newscen[currow] <- 1
        } else {
          newscen[currow] <- 0
        }
        if (implicitprePtime<=1949) {       
          newprescen[currow] <- 1
        } else {
          newprescen[currow] <- 0
        }
      }
    }
    
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }


  scenvar <- "partyexp"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- newscen[curxrow] + sd(x[,curxrow])
    newprescen <- meandata
    scenname <- "Party Exp +1 sd"

    if (any(xvar=="apptage")) {
      scenage <- newscen[(1:k)[xvar=="apptage"]]
      preage <- newprescen[(1:k)[xvar=="apptage"]]
      partyexp <- newscen[curxrow]
      partyexppre <- newprescen[curxrow]
      implicitPtime <- yr - (scenage*partyexp)
      implicitprePtime <- yr - (preage*partyexppre)
      if (any(xvar=="ccp1935")) {
        currow <-  (1:k)[xvar=="ccp1935"]
        if (implicitPtime<=1935) {       
          newscen[currow] <- 1
        } else {
          newscen[currow] <- 0
        }
        if (implicitprePtime<=1935) {       
          newprescen[currow] <- 1
        } else {
          newprescen[currow] <- 0
        }
      }
      
      if (any(xvar=="ccp1945")) {
        currow <-  (1:k)[xvar=="ccp1945"]
        if (implicitPtime<=1945) {       
          newscen[currow] <- 1
        } else {
          newscen[currow] <- 0
        }
        if (implicitprePtime<=1945) {       
          newprescen[currow] <- 1
        } else {
          newprescen[currow] <- 0
        }
      }
      
      if (any(xvar=="ccp1949")) {
        currow <-  (1:k)[xvar=="ccp1949"]
        if (implicitPtime<=1949) {       
          newscen[currow] <- 1
        } else {
          newscen[currow] <- 0
        }
        if (implicitprePtime<=1949) {       
          newprescen[currow] <- 1
        } else {
          newprescen[currow] <- 0
        }
      }
    } else {

    }
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }


  scenvar <- "partyyears"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- newscen[curxrow] + sd(x[,curxrow])
    newprescen <- meandata
    scenname <- "Party Years +1 sd"

    implicitPtime <- yr - newscen[curxrow]
    implicitprePtime <- yr - meandata[curxrow]
    
    
    if (any(xvar=="ccp1935")) {
      currow <-  (1:k)[xvar=="ccp1935"]
      if (implicitPtime<=1935) {       
        newscen[currow] <- 1
      } else {
        newscen[currow] <- 0
      }
      if (implicitprePtime<=1935) {       
        newprescen[currow] <- 1
      } else {
        newprescen[currow] <- 0
      }
    }
    
    if (any(xvar=="ccp1945")) {
      currow <-  (1:k)[xvar=="ccp1945"]
      if (implicitPtime<=1945) {       
        newscen[currow] <- 1
      } else {
        newscen[currow] <- 0
      }
      if (implicitprePtime<=1945) {       
        newprescen[currow] <- 1
      } else {
        newprescen[currow] <- 0
      }
    }
    
    if (any(xvar=="ccp1949")) {
      currow <-  (1:k)[xvar=="ccp1949"]
      if (implicitPtime<=1949) {       
        newscen[currow] <- 1
      } else {
        newscen[currow] <- 0
      }
      if (implicitprePtime<=1949) {       
        newprescen[currow] <- 1
      } else {
        newprescen[currow] <- 0 
      }
    } 
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }
  

  if (fd) {
    scenvar <- "highschool"
    if (any(xvar==scenvar)) {
      curxrow <- (1:k)[xvar==scenvar]
      newscen <- meandata
      newscen[curxrow] <- 1
      
      if (any(xvar=="college")) {
        currow <- (1:k)[xvar=="college"]
        newscen[currow] <- 0
      }
      
      if (any(xvar=="postgrad")) {
        currow <- (1:k)[xvar=="postgrad"]
        newscen[currow] <- 0
      }
    
      newprescen <- meandata
      newprescen[curxrow] <- 0
      if (any(xvar=="college")) {
        currow <- (1:k)[xvar=="college"]
        newprescen[currow] <- 0
      }
      
      if (any(xvar=="postgrad")) {
        currow <- (1:k)[xvar=="postgrad"]
        newprescen[currow] <- 0
      }
      
      scenname <- "High School vs <HS"
      newdata <- rbind(newdata,newscen)
      newpredata <- rbind(newpredata,newprescen)
      scenarios <- c(scenarios,scenname)
    }
  } else {
    scenvar <- "highschool"
    if (any(xvar==scenvar)) {
      curxrow <- (1:k)[xvar==scenvar]
      newscen <- meandata
      newscen[curxrow] <- 1
      
      if (any(xvar=="college")) {
        currow <- (1:k)[xvar=="college"]
        newscen[currow] <- 0
      }
      
      if (any(xvar=="postgrad")) {
        currow <- (1:k)[xvar=="postgrad"]
        newscen[currow] <- 0
      }
    
      newprescen <- meandata
      newprescen[curxrow] <- 0
      if (any(xvar=="college")) {
        currow <- (1:k)[xvar=="college"]
        newprescen[currow] <- 0
      }
      
      if (any(xvar=="postgrad")) {
        currow <- (1:k)[xvar=="postgrad"]
        newprescen[currow] <- 0
      }
      
      scenname <- "High School"
      newdata <- rbind(newdata,newscen)
      newpredata <- rbind(newpredata,newprescen)
      scenarios <- c(scenarios,scenname)
    }

    scenvar <- "highschool"
    if (any(xvar==scenvar)) {
      curxrow <- (1:k)[xvar==scenvar]
      newscen <- meandata
      newscen[curxrow] <- 0
      
      if (any(xvar=="college")) {
        currow <- (1:k)[xvar=="college"]
        newscen[currow] <- 0
      }
      
      if (any(xvar=="postgrad")) {
        currow <- (1:k)[xvar=="postgrad"]
        newscen[currow] <- 0
      }
    
      newprescen <- meandata
      newprescen[curxrow] <- 0
      if (any(xvar=="college")) {
        currow <- (1:k)[xvar=="college"]
        newprescen[currow] <- 0
      }
      
      if (any(xvar=="postgrad")) {
        currow <- (1:k)[xvar=="postgrad"]
        newprescen[currow] <- 0
      }
      
      scenname <- "< High School"
      newdata <- rbind(newdata,newscen)
      newpredata <- rbind(newpredata,newprescen)
      scenarios <- c(scenarios,scenname)
    }
    
  }
    

  if (fd) {
    scenvar <- "college"
    if (any(xvar==scenvar)) {
      curxrow <- (1:k)[xvar==scenvar]
      newscen <- meandata
      newscen[curxrow] <- 1
      
      if (any(xvar=="highschool")) {
        currow <- (1:k)[xvar=="highschool"]
        newscen[currow] <- 0
      }
      
      if (any(xvar=="postgrad")) {
        currow <- (1:k)[xvar=="postgrad"]
        newscen[currow] <- 0
      }
      
      newprescen <- meandata
      newprescen[curxrow] <- 0
      if (any(xvar=="highschool")) {
        currow <- (1:k)[xvar=="highschool"]
        newprescen[currow] <- 0
      }
      
      if (any(xvar=="postgrad")) {
        currow <- (1:k)[xvar=="postgrad"]
        newprescen[currow] <- 0
      }
      
      scenname <- "College vs < HS"
      newdata <- rbind(newdata,newscen)
      newpredata <- rbind(newpredata,newprescen)
      scenarios <- c(scenarios,scenname)
    }

    scenvar <- "college"
    if (any(xvar==scenvar)) {
      curxrow <- (1:k)[xvar==scenvar]
      newscen <- meandata
      newscen[curxrow] <- 1
      
      if (any(xvar=="highschool")) {
        currow <- (1:k)[xvar=="highschool"]
        newscen[currow] <- 0
      }
      
      if (any(xvar=="postgrad")) {
        currow <- (1:k)[xvar=="postgrad"]
        newscen[currow] <- 0
      }
      
      
      newprescen <- meandata
      newprescen[curxrow] <- 0
      if (any(xvar=="highschool")) {
        currow <- (1:k)[xvar=="highschool"]
        newprescen[currow] <- 1
      }
      
      if (any(xvar=="postgrad")) {
        currow <- (1:k)[xvar=="postgrad"]
        newprescen[currow] <- 0
      }
      
      scenname <- "College vs HS"
      newdata <- rbind(newdata,newscen)
      newpredata <- rbind(newpredata,newprescen)
      scenarios <- c(scenarios,scenname)
    }
  } else {
        scenvar <- "college"
    if (any(xvar==scenvar)) {
      curxrow <- (1:k)[xvar==scenvar]
      newscen <- meandata
      newscen[curxrow] <- 1
      
      if (any(xvar=="highschool")) {
        currow <- (1:k)[xvar=="highschool"]
        newscen[currow] <- 0
      }
      
      if (any(xvar=="postgrad")) {
        currow <- (1:k)[xvar=="postgrad"]
        newscen[currow] <- 0
      }
      
      newprescen <- meandata
      newprescen[curxrow] <- 0
      if (any(xvar=="highschool")) {
        currow <- (1:k)[xvar=="highschool"]
        newprescen[currow] <- 0
      }
      
      if (any(xvar=="postgrad")) {
        currow <- (1:k)[xvar=="postgrad"]
        newprescen[currow] <- 0
      }
      
      scenname <- "College"
      newdata <- rbind(newdata,newscen)
      newpredata <- rbind(newpredata,newprescen)
      scenarios <- c(scenarios,scenname)
    }

  }

  
  if (fd) {
    scenvar <- "postgrad"
    if (any(xvar==scenvar)) {
      curxrow <- (1:k)[xvar==scenvar]
      newscen <- meandata
      newscen[curxrow] <- 1
      
      if (any(xvar=="highschool")) {
        currow <- (1:k)[xvar=="highschool"]
        newscen[currow] <- 0
      }
      
      if (any(xvar=="college")) {
        currow <- (1:k)[xvar=="college"]
        newscen[currow] <- 0
      }
      
      
      newprescen <- meandata
      newprescen[curxrow] <- 0
      if (any(xvar=="highschool")) {
        currow <- (1:k)[xvar=="highschool"]
        newprescen[currow] <- 0
      }
      
      if (any(xvar=="college")) {
        currow <- (1:k)[xvar=="college"]
        newprescen[currow] <- 0
      }
      
      scenname <- "Graduate School vs < HS"
      newdata <- rbind(newdata,newscen)
      newpredata <- rbind(newpredata,newprescen)
      scenarios <- c(scenarios,scenname)
    }
    
    scenvar <- "postgrad"
    if (any(xvar==scenvar)) {
      curxrow <- (1:k)[xvar==scenvar]
      newscen <- meandata
      newscen[curxrow] <- 1
      
      if (any(xvar=="highschool")) {
        currow <- (1:k)[xvar=="highschool"]
        newscen[currow] <- 0
      }
      
      if (any(xvar=="college")) {
        currow <- (1:k)[xvar=="college"]
        newscen[currow] <- 0
      }
            
      newprescen <- meandata
      newprescen[curxrow] <- 0
      if (any(xvar=="highschool")) {
        currow <- (1:k)[xvar=="highschool"]
        newprescen[currow] <- 0
      }
      
      if (any(xvar=="college")) {
        currow <- (1:k)[xvar=="college"]
        newprescen[currow] <- 1
      }
      
      scenname <- "Graduate School vs College"
      newdata <- rbind(newdata,newscen)
      newpredata <- rbind(newpredata,newprescen)
      scenarios <- c(scenarios,scenname)
    }
  } else {
        scenvar <- "postgrad"
    if (any(xvar==scenvar)) {
      curxrow <- (1:k)[xvar==scenvar]
      newscen <- meandata
      newscen[curxrow] <- 1
      
      if (any(xvar=="highschool")) {
        currow <- (1:k)[xvar=="highschool"]
        newscen[currow] <- 0
      }
      
      if (any(xvar=="college")) {
        currow <- (1:k)[xvar=="college"]
        newscen[currow] <- 0
      }
      
      
      newprescen <- meandata
      newprescen[curxrow] <- 0
      if (any(xvar=="highschool")) {
        currow <- (1:k)[xvar=="highschool"]
        newprescen[currow] <- 0
      }
      
      if (any(xvar=="college")) {
        currow <- (1:k)[xvar=="college"]
        newprescen[currow] <- 0
      }
      
      scenname <- "Graduate School"
      newdata <- rbind(newdata,newscen)
      newpredata <- rbind(newpredata,newprescen)
      scenarios <- c(scenarios,scenname)
    } 
  }



  scenvar <- "maodummy"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- 1
    newprescen <- meandata
    newprescen[curxrow] <- 0
    scenname <- "Mao Faction"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "dengdummy"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- 1
    newprescen <- meandata
    newprescen[curxrow] <- 0
    scenname <- "Deng Faction"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "huyaobangdummy"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- 1
    newprescen <- meandata
    newprescen[curxrow] <- 0
    scenname <- "Hu Yaobang Faction"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "chenyundummy"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- 1
    newprescen <- meandata
    newprescen[curxrow] <- 0
    scenname <- "Chen Yun Faction"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "yejianyingdummy"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- 1
    newprescen <- meandata
    newprescen[curxrow] <- 0
    scenname <- "Ye Jianying Faction"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "zhaoziyangdummy"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- 1
    newprescen <- meandata
    newprescen[curxrow] <- 0
    scenname <- "Zhao Ziyang Faction"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }


  scenvar <- "jiangzemindummy"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- 1
    newprescen <- meandata
    newprescen[curxrow] <- 0
    scenname <- "Jiang Zemin Faction"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "hujintaodummy"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- 1
    newprescen <- meandata
    newprescen[curxrow] <- 0
    scenname <- "Hu Jintao Faction"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }
  


  scenvar <- "farmy2"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- 1
    newprescen <- meandata
    newprescen[curxrow] <- 0
    scenname <- "2nd Field Army Vet"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "lmarch"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- 1
    newprescen <- meandata
    newprescen[curxrow] <- 0
    scenname <- "Long Marcher"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }
  
  scenvar <- "gdp"
  if (any(xvar=="cntminexp")&&any(xvar=="cmXgdp")) {
    if (any(xvar==scenvar)) {
    
      curxrow <- (1:k)[xvar==scenvar]
      newscen <- meandata
      cond <- (x[,curxrow]!=0)
      newscen[curxrow] <- mean(x[cond,curxrow]) + sd(x[cond,curxrow])
      newprescen <- meandata
      newprescen[curxrow] <- mean(x[cond,curxrow])
      
      if (any(xvar=="provleader")) {
        currow <- (1:k)[xvar=="provleader"]
        newscen[currow] <- 1
        newprescen[currow] <- 1
      }
      
      cur1row <- (1:k)[xvar=="cntminexp"]
      newscen[cur1row] <- 0
      newprescen[cur1row] <- 0

      cur2row <- (1:k)[xvar=="cmXgdp"]
      newscen[cur2row] <- newscen[cur1row]*newscen[curxrow]
      newprescen[cur2row] <- newprescen[cur1row]*newscen[curxrow]
            
      scenname <- "Rel. GDP Growth (No State Exp) +1 sd"
      newdata <- rbind(newdata,newscen)
      newpredata <- rbind(newpredata,newprescen)
      scenarios <- c(scenarios,scenname)   

      newscen[cur1row] <- 0.5
      newprescen[cur1row] <- 0.5
      newscen[cur2row] <- newscen[cur1row]*newscen[curxrow]
      newprescen[cur2row] <- newprescen[cur1row]*newprescen[curxrow]
            
      scenname <- "Rel. GDP Growth (50% State Exp) +1 sd"
      newdata <- rbind(newdata,newscen)
      newpredata <- rbind(newpredata,newprescen)
      scenarios <- c(scenarios,scenname)
      
      
    }
  } else {
    curxrow <- (1:k)[xvar==scenvar]
      newscen <- meandata
      cond <- (x[,curxrow]!=0)
      newscen[curxrow] <- mean(x[cond,curxrow]) + sd(x[cond,curxrow])
      newprescen <- meandata
      newprescen[curxrow] <- mean(x[cond,curxrow])
      
      if (any(xvar=="provleader")) {
        currow <- (1:k)[xvar=="provleader"]
        newscen[currow] <- 1
        newprescen[currow] <- 1
      }
      
      scenname <- "Relative GDP Growth +1 sd"
      newdata <- rbind(newdata,newscen)
      newpredata <- rbind(newpredata,newprescen)
      scenarios <- c(scenarios,scenname)
  }




  
  scenvar <- "fisrevenue"
  if (any(xvar=="cntminexp")&&any(xvar=="cmXfisrev")) {
    if (any(xvar==scenvar)) {
    
      curxrow <- (1:k)[xvar==scenvar]
      newscen <- meandata
      cond <- (x[,curxrow]!=0)
      newscen[curxrow] <- mean(x[cond,curxrow]) + sd(x[cond,curxrow])
      newprescen <- meandata
      newprescen[curxrow] <- mean(x[cond,curxrow])
      
      if (any(xvar=="provleader")) {
        currow <- (1:k)[xvar=="provleader"]
        newscen[currow] <- 1
        newprescen[currow] <- 1
      }
      
      cur1row <- (1:k)[xvar=="cntminexp"]
      newscen[cur1row] <- 0
      newprescen[cur1row] <- 0

      cur2row <- (1:k)[xvar=="cmXfisrev"]
      newscen[cur2row] <- newscen[cur1row]*newscen[curxrow]
      newprescen[cur2row] <- newprescen[cur1row]*newscen[curxrow]

      if (any(xvar=="statecurrent")) {
        currow <- (1:k)[xvar=="statecurrent"]
        newscen[currow] <- 0
        newprescen[currow] <- 0
      }
            
      scenname <- "Rel. Fisc. Growth (No State Exp) +1 sd"
      newdata <- rbind(newdata,newscen)
      newpredata <- rbind(newpredata,newprescen)
      scenarios <- c(scenarios,scenname)   

      newscen[cur1row] <- 0.5
      newprescen[cur1row] <- 0.5
      newscen[cur2row] <- newscen[cur1row]*newscen[curxrow]
      newprescen[cur2row] <- newprescen[cur1row]*newprescen[curxrow]
      
      if (any(xvar=="statecurrent")) {
        currow <- (1:k)[xvar=="statecurrent"]
        newscen[currow] <- mean(x[,currow])
        newprescen[currow] <- mean(x[,currow])
      }
            
      scenname <- "Rel. Fisc. Growth (50% State Exp) +1 sd"
      newdata <- rbind(newdata,newscen)
      newpredata <- rbind(newpredata,newprescen)
      scenarios <- c(scenarios,scenname)
      
      
    }
  } else {
    curxrow <- (1:k)[xvar==scenvar]
      newscen <- meandata
      cond <- (x[,curxrow]!=0)
      newscen[curxrow] <- mean(x[cond,curxrow]) + sd(x[cond,curxrow])
      newprescen <- meandata
      newprescen[curxrow] <- mean(x[cond,curxrow])
      
      if (any(xvar=="provleader")) {
        currow <- (1:k)[xvar=="provleader"]
        newscen[currow] <- 1
        newprescen[currow] <- 1
      }
      
      scenname <- "Relative Fiscal Growth +1 sd"
      newdata <- rbind(newdata,newscen)
      newpredata <- rbind(newpredata,newprescen)
      scenarios <- c(scenarios,scenname)
  }


  scenvar <- "placurrent"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- 1
    newprescen <- meandata
    newprescen[curxrow] <- 0
    scenname <- "PLA"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "localgovcurrent"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- 1
    newprescen <- meandata
    newprescen[curxrow] <- 0
    scenname <- "Local Gov't"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "statecurrent"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- 1
    newprescen <- meandata
    newprescen[curxrow] <- 0
    scenname <- "State Council"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "centralpartycurrent"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- 1
    newprescen <- meandata
    newprescen[curxrow] <- 0
    scenname <- "Central Party"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "provleader"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- 1
    newprescen <- meandata
    newprescen[curxrow] <- 0

    if (any(xvar=="fisrevenue")) {
      currow <- (1:k)[xvar=="fisrevenue"]
      cond <- (x[,curxrow]!=0)
      newscen[currow] <- mean(x[cond,currow])
      newprescen[currow] <- 0
    }

    if (any(xvar=="gdp")) {
      currow <- (1:k)[xvar=="gdp"]
      cond <- (x[,curxrow]!=0)
      newscen[currow] <- mean(x[cond,currow])
      newprescen[currow] <- 0
    }
    
    scenname <- "Provincial Leader"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  curxrow <- (1:k)[xvar==scenvar]
  newscen <- meandata
  newprescen <- meandata
  scenname <- "Average Member"
  newdata <- rbind(newdata,newscen)
  newpredata <- rbind(newpredata,newprescen)
  scenarios <- c(scenarios,scenname)
  

  #newdata <- as.data.frame(newdata)
  #newpredata <- as.data.frame(newpredata)
  rownames(newdata) <- scenarios
  rownames(newpredata) <- scenarios
  
  result <- list(x=newdata,xpre=newpredata,scenarios=scenarios)
  result
}


firstdiffTable <- function(filename,fd,n,scen) {

  fd$lower <- t(fd$lower)
  fd$upper <- t(fd$upper)
  
  rankfd <- fd$rank
  pctfd <- -100*fd$rank/n
  lowerfd <- fd$lower
  upperfd <- fd$upper
  lowerpct <- -100*fd$lower/n
  upperpct <- -100*fd$upper/n

  result <- cbind(scen,rankfd,pctfd,lowerfd,upperfd,lowerpct,upperpct)
  result <- as.data.frame(result)
  names(result) <- c("case","x","px","lowerx1","lowerx2","upperx1","upperx2","lowerpx1","lowerpx2","upperpx1","upperpx2")

  write.csv(result,paste(filename,"csv",sep="."))

}

paramtable <- function(filename,result,varnames) {
  tab <- result$summary
  ltab <- nrow(tab)
  lx <- length(varnames)
  lcol <- c(1,2,3,7,8)
  param <-result$summary[ (ltab - lx) : (ltab - 1), lcol]
  rownames(param) <- varnames
  BIC.AIC.DIC.dev.R2 <- c(result$BIC,result$AIC,result$DIC,result$deviance,result$R2)
  param <- rbind(param,BIC.AIC.DIC.dev.R2)
  print(param)
  write.csv(param,paste(filename,"csv",sep="."))
}


agepartyCF <- function(x,xvar,yr,ptime,apptage) {

  # Return new data and scenario names
  k <- length(xvar)
  meandata <- apply(x,2,mean)

  # Ensure logical consistency
  if (any(xvar=="apptage")) {
    curxrow <- (1:k)[xvar=="apptage"]
    if (meandata[curxrow]>=63) {
      if (any(xvar=="agege63")) {
        currow <- (1:k)[xvar=="agege63"]   
        meandata[currow] <- 1
      }
    } else {
      if (any(xvar=="agege63")) {
        currow <- (1:k)[xvar=="agege63"]   
        meandata[currow] <- 0
      }
    }

    meanage <- mean(x[,curxrow])

    if (any(xvar=="partyexp")) {
      partyexp <- mean(x[,(1:k)[xvar=="partyexp"]])
      implicitPtime <- yr - (meanage*partyexp)
      if (any(xvar=="ccp1935")) {
        currow <-  (1:k)[xvar=="ccp1935"]
        if (implicitPtime<=1935) {       
          meandata[currow] <- 1
        } else {
          meandata[currow] <- 0
        }
      }
      
      if (any(xvar=="ccp1945")) {
        currow <-  (1:k)[xvar=="ccp1945"]
        if (implicitPtime<=1945) {       
          meandata[currow] <- 1
        } else {
          meandata[currow] <- 0
        }
      }
      
      if (any(xvar=="ccp1949")) {
        currow <-  (1:k)[xvar=="ccp1949"]
        if (implicitPtime<=1949) {       
          meandata[currow] <- 1
        } else {
          meandata[currow] <- 0
        }
      }    
    }    
  }

  if (any(xvar=="partyyears")) {
    implicitPtime <- yr - mean(x[,(1:k)[xvar=="partyyears"]])
    if (any(xvar=="ccp1935")) {
      currow <-  (1:k)[xvar=="ccp1935"]
      if (implicitPtime<=1935) {       
        meandata[currow] <- 1
      } else {
        meandata[currow] <- 0
      }
    }
    
    if (any(xvar=="ccp1945")) {
      currow <-  (1:k)[xvar=="ccp1945"]
      if (implicitPtime<=1945) {       
        meandata[currow] <- 1
      } else {
        meandata[currow] <- 0
      }
    }
    
    if (any(xvar=="ccp1949")) {
      currow <-  (1:k)[xvar=="ccp1949"]
      if (implicitPtime<=1949) {       
        meandata[currow] <- 1
      } else {
        meandata[currow] <- 0
      }
    }    
  }
  
  newdata <- NULL

  minptime <- min(na.omit(ptime))
  maxptime <- max(na.omit(ptime))
  minage <- min(na.omit(apptage))
  maxage <- max(na.omit(apptage))
  
  hptime <- seq(minptime,maxptime)
  hage <- seq(minage,maxage)

  hptimeAll <- hageAll <- hpartyexpAll <- NULL
  for (i in 1:length(hptime)) {
    for (j in 1:length(hage)) {

      hptimeAll <- c(hptimeAll,hptime[i])
      hageAll <- c(hageAll,hage[j])
      hpartyexpAll <- c(hpartyexpAll,  (yr-hptime[i])/hage[j] )
      
      newscen <- meandata
      
      scenvar <- "apptage"
      if (any(xvar==scenvar)) {
        curxrow <- (1:k)[xvar==scenvar]  
        newscen[curxrow] <- hage[j]
      }

      if (any(xvar=="agege63")) {
        currow <- (1:k)[xvar=="agege63"]
        if (hage[j]>=63) {
          newscen[currow] <- 1
        } else {
          newscen[currow] <- 0
        }
      }

      scenvar <- "partyexp"
      if (any(xvar==scenvar)) {
        curxrow <- (1:k)[xvar==scenvar]
        newscen[curxrow] <- (yr-hptime[i])/hage[j]
      }

      
      if (any(xvar=="ccp1935")) {
        currow <-  (1:k)[xvar=="ccp1935"]
        if (hptime[i]<=1935) {       
          newscen[currow] <- 1
        } else {
          newscen[currow] <- 0
        }
      }
      
      if (any(xvar=="ccp1945")) {
        currow <-  (1:k)[xvar=="ccp1945"]
        if (hptime[i]<=1945) {       
          newscen[currow] <- 1
        } else {
          newscen[currow] <- 0
        }
      }
      
      if (any(xvar=="ccp1949")) {
        currow <-  (1:k)[xvar=="ccp1949"]
        if (hptime[i]<=1949) {       
          newscen[currow] <- 1
        } else {
          newscen[currow] <- 0
        }
      }
      
      newdata <- rbind(newdata,newscen)
    }
  }
  
  result <- list(newdata=newdata,hypptime=hptimeAll,hypage=hageAll,hyppartyexp=hpartyexpAll)
  result
}





